#!/usr/bin/python

import sys
from qt import *
from kdecore import *

from rssmixtape.Config import config

from rssmixtape.MixTapeImpl import MixTapeImpl
from rssmixtape.ConfigEditorImpl import ConfigEditorImpl
from rssmixtape.FeedEditorImpl import FeedEditorImpl
from rssmixtape.DBConvert import canConvert, convert

from rssmixtape import format


def main (args):
    app = KApplication (args, "mixtape")

    #convert outdated databases
    dbversion = config ().get ("main", "version")
    if dbversion != format and config ().get ("rss", "outputDir"):

        if canConvert (dbversion, format):
            
            x = QMessageBox.warning (
                None, "RSS Mix Tape",
                "Your configuration is from a different version of \n" +
                "RSS Mix Tape.  Shall I convert it to this version? \n" +
                "I think I can succeed, and I will try not to mess up \n" +
                "your data if I fail.",
                "Convert",
                "Don't Convert", None, 0, 1 )

            if x == 0:
                if not convert (dbversion, format):
                    QMessageBox.critical (None, "RSS Mix Tape",
                        "Unfortunately, I failed to convert your " +
                        "configuration.  Your data should not have not " +
                        "been altered.  Please contact the author.",
                        QMessageBox.Ok, QMessageBox.NoButton)
                    
                    return
                else:
                    #success
                    config ().set ("main", "version", format)
                    config ().save ()

            else:
                QMessageBox.critical (None, "RSS Mix Tape",
                        "I can't continue until you convert your data.",
                        QMessageBox.Ok, QMessageBox.NoButton)

                return

    if not config ().get ("rss", "outputDir"):
        cei = ConfigEditorImpl ()
        cei.exec_loop ()
        if cei.result () == QDialog.Rejected:
            return #not configured
        else:
            config ().set ("main", "version", format)
            config ().save ()
            QMessageBox.information (
                None, "RSS Mix Tape",
                "Now, add some feeds -- when you're done, click \"Done\" to" +
                "continue to the main window.",
                QMessageBox.Ok)
                
            fei = FeedEditorImpl ()
            fei.exec_loop ()

            
    mti = MixTapeImpl ()
    mti.show ()
    
    app.connect (app, SIGNAL ("lastWindowClosed ()"),
                app, SLOT ("quit ()"))
    app.exec_loop ()
    
if __name__ == "__main__":
    main (sys.argv)
