#!/usr/bin/python

# RSS Mix Tape
# Copyright 2003, David "Novalis" Turner <novalis@fsf.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

from qt import *

from ConfigEditor2 import ConfigEditor
import Interval

import Config
import re

class ConfigEditorImpl (ConfigEditor):
    def __init__ (self):
        ConfigEditor.__init__ (self)
        secs = Interval.revtimes.keys ()
        secs.sort ()
        self.minUpdate.insertStrList ([Interval.revtimes [x] for x in secs])

        config = Config.config ()
        #set up the combobox for the minimum update interval
        mi = config.get ("rss", "minUpdate")
        if not mi:
            print "default mi"
            #The default should not expire an item in 15 minutes
            self.minUpdate.setCurrentItem (6)
            config.set ("rss", "minUpdate",
                        Interval.times
                        [str (self.minUpdate.currentText ())])
            
        else:
            i = 0
            for s in secs:
                if int (mi) == s:
                    self.minUpdate.setCurrentItem (i)
                    continue

                i += 1

        for attr in ("link", "title", "description", "outputDir",
                     "managingEditor"):
            value = config.get ("rss", attr)
            if value:
                getattr (self, attr).setText (value)

        ljUser = config.get ("lj", "user")
        if ljUser:
            self.ljUser.setText (ljUser)

        ljPass = config.get ("lj", "pass")
        if ljPass:
            self.ljPass.setText (ljPass)


        qtUsername = config.get ("quicktopic", "username")
        if qtUsername:
            self.qtUsername.setText (qtUsername)

        qtEmail = config.get ("quicktopic", "email")
        if qtEmail:
            self.qtEmail.setText (qtEmail)
            
        rx = QRegExp ("[^/]+")

        self.newCategory.setValidator (QRegExpValidator (rx, self.newCategory))
        self.cats = config.getCategories ()
        self.categories.insertStrList (self.cats)

    def add_category (self):
        text = str (self.newCategory.text ())
        if len (text):
            self.cats.append (text)
            self.categories.insertItem (text)
            self.newCategory.setText ("")

    def delete_category (self):
        i = self.categories.currentItem ()
        if i == -1:
            return
        self.categories.removeItem (i)
        self.cats = self.cats [0:i] + self.cats [i + 1:-1]

    def accept (self):
        config = Config.config ()
        config.set ("rss", "minUpdate",
                    str (Interval.times [str (self.minUpdate.currentText ())]))
        config.set ("rss", "title", str (self.title.text ()))
        config.set ("rss", "link", str (self.link.text ()))
        config.set ("rss", "description", str (self.description.text ()))
        config.set ("rss", "outputDir", str (self.outputDir.text ()))

        config.set ("lj", "user", str (self.ljUser.text ()))
        config.set ("lj", "pass", str (self.ljPass.text ()))

        config.set ("quicktopic", "username", str (self.qtUsername.text ()))
        config.set ("quicktopic", "email", str (self.qtEmail.text ()))
        
        config.setCategories (self.cats)
        config.save ()
        ConfigEditor.accept (self)
