#!/usr/bin/python

# RSS Mix Tape
# Copyright 2003, David "Novalis" Turner <novalis@fsf.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

from Config import config, dir
from cPickle import dumps
from os import rename

from LockFile import LockFile
from bsddb import *

def canConvert (v1, v2):
    if v1 or v2 != '1.2.0':
        print "can't convert post from ", v1, " to ", v2
    return (not v1) and v2 == '1.2.0'

def convertPostDB (v1, v2, id):
    if v1 or v2 != '1.2.0':
        print "can't convert post from ", v1, " to ", v2
        return 0

    filename = dir + "feed/" + str (id) + ".db"
    db = btopen (filename, "c")
    newdb = btopen (filename + '.new', "c")
    
    posts = []
    try:
        k, v = db.first ()
        posts.append (makePostDictV0 (k, v))
        while 1:
            k, v = db.next ()
            posts.append (makePostDictV0 (k, v))
    except KeyError:
        pass

    for post in posts:
        id = post ['id']
        del post ['id']
        newdb [id] = dumps (post)
        
    db.close ()
    newdb.close ()

    return 1

def convert (v1, v2):
    if v1 or v2 != '1.2.0':
        print "can't convert from ", v1, " to ",v2
        return 0
    
    lock = LockFile (dir + "feedlist.lock")
    if lock.locked ():
        raise "Locked"
    lock.lock ()

    filename = dir + "feedlist"
    db = hashopen (filename, "c")
    newdb = hashopen (filename + '.new', "c")
    
    feeds = []
    try:
        k, v = db.first ()
        feeds.append (makeFeedDictV0 (k, v))
        while 1:
            k, v = db.next ()
            feeds.append (makeFeedDictV0 (k, v))
    except KeyError:
        pass

    try:
        for feed in feeds:
            id = feed ['id']
            del feed ['id']
            newdb [id] = dumps (feed)
            feed ['id'] = id
            if not convertPostDB (v1, v2, id):
                raise "Failed to convert post db"
            
        db.close ()
        newdb.close ()
        for feed in feeds:
            id = feed ['id']
            filename = dir + "feed/" + str (id) + ".db"
            rename (filename + '.new', filename)

        rename (dir + 'feedlist.new', dir + 'feedlist')

        status = 1
    except:
        db.close ()
        newdb.close ()
        status = 0
        raise
    
    lock.unlock ()
        
    return status


def makeFeedDictV0 (k, v):
    args = v.split ("\0")
    feed = {}
    feed ['title'], feed ['interval'], feed ['url'], feed ['lastchecked'] = args
    feed ['id'] = k
    feed ['deleted'] = 0
    for x in 'interval', 'lastchecked':
        feed [x] = int (feed [x])
    return feed

def makePostDictV0 (k, v):
    args = v.split ("\0")
    post = {}
    post ['feed'], post ['date'], post ['title'], post ['link'], post ['text'], post ['comment'], post ['posted'], post ['deleted'], post ['generated'] = args
    post ['id'] = k
    for x in 'date', 'deleted', 'generated':
        post [x] = int (post [x])

    return post

