#!/usr/bin/python

# RSS Mix Tape
# Copyright 2003, David "Novalis" Turner <novalis@fsf.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

from __future__ import generators

from bsddb import *
import time
import os
from cPickle import loads, dumps

import DateTime

import rssparser

from Config import config, dir
from Post2 import Post

from LiveJournal import ljContext

class Feed:
    def __init__ (self, args):
        """Create a new feed"""
        for k, v in args.items ():
            setattr (self, k, v)
            
        for k in 'lastchecked', 'deleted', 'id':
            if not k in args:
                setattr (self, k, 0)
                
        if not self.id:
            self.id = config ().counter ()
        self.id = str (self.id)
        
        self.recentPosts = None
        self.listener = None
        self.posts = None
        self.db = None
        self.deadList = []
        self.postCacheLen = 50
        
    def toValue (self):
        d = {}
        for var in 'title', 'interval', 'url', 'lastchecked', 'deleted':
            d [var] = getattr (self, var)
        return dumps (d)

    def setListener (self, listener):
        self.listener = listener

    def open (self):
        if self.db is not None:
            return
        
        if not os.path.exists (dir + "feed/"):
            os.mkdir (dir + "feed/")

        self.db = btopen (dir + "feed/" + self.id + ".db", "c")

    def update (self):
        """Updates the data in the feed from the url"""

        self.open ()

        #get the N most recent posts, if we haven't already got them
        if not self.recentPosts:
            i = 0
            self.recentPosts = []
            try:
                k, v = self.db.last ()
                self.recentPosts.append (self.createPost (k, v))
                while 1:
                    if i >= self.postCacheLen:
                        break
                    k, v = self.db.previous () 
                    self.recentPosts.append (self.createPost (k, v))
            except KeyError:
                pass


        self.getPostsFromRSS ()

    def getPostsFromRSS (self):

        now = int (time.time ())
        self.lastchecked = now

        cookies = None

        if self.url.startswith ('lj user='):
            url = 'http://www.livejournal.com/customview.cgi?styleid=90448&username='
            url += self.url [self.url.index ('=') + 1:] #the stuff after =
            cookie = ljContext ().login ()
            if cookie:
                cookies = [cookie]
            else:
                cookies = None
        else:
            url = self.url


        rss = rssparser.parse (url, None, None, None, None, cookies=cookies)

        items = rss ["items"]
        channel = rss ["channel"]

        for item in items:
            self.feedItem (item, channel, now)

    def feedItem (self, item, channel, now):
        if not 'date' in item:
            if 'date' in channel:
                item ['date'] = channel ['date']
            else:
                if 'lastBuildDate' in channel:
                    item ['date'] = channel ['lastBuildDate']
                else:
                    item ['date'] = time.asctime (time.localtime (now))

        try:
            item ['date'] = int (DateTime.DateTime (item ['date']).timeTime ())
        except:
            #The Penny-Arcade feed triggers this one
            item ['date'] = int (time.time ())


        if not 'title' in item:
            if 'description' in item:
                item ['title'] = item ['description']
            else: 
                if 'title' in channel:
                    item ['title'] = channel ['title']
                else:
                    item ['title'] = 'untitled'

        if len (item ['title']) > 50:
            item ['title'] = item ['title'][0:50] + "..."

        if not 'text' in item:
            if 'description' in item:
                item ['text'] = item ['description']
            else:
                item ['text'] = ''

        if not 'link' in item:
            if 'link' in channel:
                item ['link'] = channel ['link']
            else:
                item ['link'] = ''

        item ['feed'] = self.title
        item ['id'] = 0

        #it would be a security risk to allow these to be set from the RSS
        #feed
        for attr in ('createQT', 'deleted', 'generated'):
            item [attr] = 0
        item ['comment'] = ''
        item ['posted'] = 'not posted'

        post = Post (item)
        self.addPost (post)        

    def purge (self):
        for post in self.deadList:
            try:
                del self.db [post.id]
            except KeyError:
                pass #fixme: shouldn't be here
                
            self.posts.remove (post)

        self.deadList = []

    def sync (self):
        if self.posts:
            for post in self.posts:
                self.db [post.id] = post.toValue ()
        self.db.sync ()

    def close (self):
        if self.db is not None:
            self.purge ()
            if self.posts:
                for post in self.posts:
                    self.db [post.id] = post.toValue ()
            self.db.close ()
            self.db = None

    def readPosts (self):
        if self.posts:
            for post in self.posts:
                self.listener (post)
            return
                
        self.posts = []
        try:
            k, v = self.db.first ()
            self.addPost (self.createPost (k, v), 0)
            while 1:
                k, v = self.db.next ()
                self.addPost (self.createPost (k, v), 0)
        except KeyError:
            return
        
    def forPosts (self, fun):
        if not self.posts:
            self.readPosts ()
        for post in self.posts:
            fun (post)

    def addPost (self, post, check = 1):
        """The check variable determines if this is a new post from
        the feed (1), or a stored post from the db"""

        if not self.recentPosts:
            self.recentPosts = []
            
        if check:
            for oldpost in self.recentPosts:
                if oldpost.title == post.title and oldpost.text == post.text:
                    return 0

		#on the Volokh Conspiracy, old posts become title-only
                if oldpost.title == post.title and len (oldpost.text) > 7 and not post.text:
                    return 0
            self.db [post.id] = post.toValue ()
        else:
            #post from DB
            #might want to add it to the deadlist
            if post.deleted and time.time () - \
                   post.deleted > int (config ().get ("rss", "minUpdate")):
                self.deadList.append (post)


        self.recentPosts.append (post)
        if len (self.recentPosts) > self.postCacheLen:
            self.recentPosts = self.recentPosts [1:]

        if not self.posts:
            self.posts = []
        self.posts.append (post)
        if self.listener:
            self.listener (post)

    def delete (self):
        self.deleted = 1

    def createPost (self, k, v):
        """Creates a post from a database key and value"""
        args = loads (v)
        args ['id'] = k
        return Post (args)

    def needsUpdate (self):
        """Returns true if the feed should be updated from its RSS source"""
        return self.lastchecked + self.interval < time.time ()
        
