#!/usr/bin/python

# RSS Mix Tape
# Copyright 2003, David "Novalis" Turner <novalis@fsf.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


from __future__ import generators
from bsddb import *
from cPickle import loads, dumps

from LockFile import *

from Config import config, dir
from Feed2 import Feed


def feedDB ():
    if not FeedDB._inst:
        lock = LockFile (dir + "feedlist.lock")
        if lock.locked ():
            raise "Locked"
        FeedDB._inst = FeedDB ()
    return FeedDB._inst

        
class FeedDB:
    _inst = None
    def __init__ (self):
        self.lock = LockFile (dir + "feedlist.lock")
        self.lock.lock ()
        self.listeners = []
        self.feeds = None
        self.open ()
        
    def open (self):
        self.db = hashopen (dir + "feedlist", "c")

    def addListener (self, listener):
        self.listeners.append (listener)

    def __del__ (self):
        self.close ()


    def close (self):
        for feed in self.feeds:
            self.store (feed)
            feed.close ()
        self.db.close ()
        try:
            self.lock.unlock ()
        except NotLockedError:
            pass
            
        self.db = None
        
    def feedList (self):
        if self.feeds:
            return self.feeds
        self.feeds = []
        try:
            k, v = self.db.first ()
            self.feeds.append (self.createFeed (k, v))
            while 1:
                k, v = self.db.next ()
                self.feeds.append (self.createFeed (k, v))
        except KeyError:
            pass

        return self.feeds

    def createFeed (self, k, v):
        """Creates a feed from a database key and value"""
        args = loads (v)
        args ['id'] = k
        return Feed (args)

    def addFeed (self, feed):
        self.db [str (feed.id)] = feed.toValue ()
        self.feeds.append (feed)

        for listener in self.listeners:
            listener (feed)

    def delete (self, feed):
        feed.delete ()

    def getLiveFeeds (self):

        return filter (lambda feed: not feed.deleted, self.feeds)
            
    def store (self, feed):
        """Store the feed's current state in the feed db. """

        if not self.db:
            self.open ()

        if feed.deleted and not feed.posts:
            del self.db [str (feed.id)]
            self.feeds.remove (feed)
        else:
            self.db [str (feed.id)] = feed.toValue ()
    
    def posts (self):
        for feed in self.feeds:
            for post in feed.posts:
                yield post
