#!/usr/bin/python

# RSS Mix Tape
# Copyright 2003, David "Novalis" Turner <novalis@fsf.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


from qt import *
from qttable import *

import time

from NewFeedImpl import NewFeedImpl
from FeedEditor import FeedEditor
from FeedDB2 import feedDB
from FeedLoader import feedLoader
from Feed2 import Feed

import Interval

class FeedEditorImpl (FeedEditor):
    def __init__ (self, parent = None):
        FeedEditor.__init__ (self, parent)
        table = self.feedTable
        header = table.horizontalHeader ();
        header.setLabel (0, "Title")
        header.setLabel (1, "Last Checked")
        header.setLabel (2, "Interval")
        header.setLabel (3, "URL")

        table.setSelectionMode (QTable.SingleRow)

        feeds = feedDB ().feedList ()
        feedDB ().addListener (self.addFeed)
        table.setNumRows (len (feeds))
        #get the feed list into the table
        row = 0
        for feed in feeds:
            self.addFeed (feed, row)
            row += 1
            
        self.feedTable.setLeftMargin (0)
        self.feedTable.verticalHeader ().hide ()
        feedLoader ().setListener (self.feedUpdated)

    def closeEvent (self, event):
        feedLoader ().setListener (None)
        FeedEditor.closeEvent (self, event)

    def addFeed (self, feed, row = -1):
        if feed.deleted: return
        table = self.feedTable
        if row == -1:
            row = table.numRows ()
            table.setNumRows (row + 1)

        titleItem = QTableItem (table, QTableItem.WhenCurrent, feed.title)
        table.setItem (row, 0, titleItem)

        if feed.lastchecked:
            strdate = time.asctime (time.localtime (feed.lastchecked))
        else:
            strdate = "Never"
        lcItem = QTableItem (table, QTableItem.Never, strdate)
        table.setItem (row, 1, lcItem)
            
        interval = TimeDropDown (self.feedTable, feed)
        table.setItem (row, 2, interval)
            
        urlItem = QTableItem (table, QTableItem.WhenCurrent, feed.url)
        table.setItem (row, 3, urlItem)
        
        for i in range (0, 4):
            table.adjustColumn (i)

    def delete_feed (self):
        table = self.feedTable
        row = table.currentRow ()
        if row >= 0 and row < table.numRows ():
            feed = table.item (row, 2).feed ()
            feedDB ().delete (feed)
            table.removeRow (row)

    def update_all_feeds (self):
        feedLoader ().loadAll ()
        
    def update_feed (self):
        table = self.feedTable
        row = table.currentRow ()
        if row >= 0 and row < table.numRows ():
            feed = table.item (row, 2).feed ()
            feedLoader ().load (feed)

    def feedUpdated (self, feed):
        strdate = time.asctime (time.localtime (feed.lastchecked))
        lcItem = QTableItem (self.feedTable, QTableItem.Never, strdate)
        #find out where this goes
        for row in range (0, self.feedTable.numRows ()):
            if feed == self.feedTable.item (row, 2).feed ():
                self.feedTable.setItem (row, 1, lcItem)
                break
 
    def new_feed (self):
        nfi = NewFeedImpl (self)
        nfi.show ()

    def accept (self):
        for row in range (0, self.feedTable.numRows ()):
            feed = self.feedTable.item (row, 2).feed ()
            feed.title = str (self.feedTable.text (row, 0))
            feed.url = str (self.feedTable.text (row, 3))
            feedDB ().store (feed)
        QDialog.accept (self)

    def value_changed (self):
        table = self.feedTable
        row = table.currentRow ()
        tdd = table.item (row, 2)
        tdd.updateFeed ()
        
class TimeDropDown (QComboTableItem):
    def __init__ (self, parent, feed):
        ivals = Interval.times.values ()
        ivals.sort ()
        ivals = QStringList.fromStrList ([Interval.revtimes [x] for x in ivals])
        QComboTableItem.__init__ (self, parent, ivals)
        self._feed = feed
        self.setCurrentItem (Interval.revtimes [feed.interval])

    def feed (self):
        return self._feed

    def set (self, i):
        self.setCurrentItem (i)
        self._feed.interval = Interval.times [str (i)]

    def updateFeed (self):
        self._feed.interval = Interval.times [str (self.currentText ())]


#need to handle: inserting new rows when new blogs are fully added
#special-case dates of 0
