#!/usr/bin/python

# RSS Mix Tape
# Copyright 2003, David "Novalis" Turner <novalis@fsf.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

from threading import *

from qt import QObject

from FeedDB2 import feedDB

def feedLoader ():
    if not FeedLoader._inst:
        FeedLoader._inst = FeedLoader ()
    return FeedLoader._inst

class FeedLoader (QObject):
    _inst = None
    def __init__ (self):
        QObject.__init__ (self)
        self.threads = []
        self.queue = []
        self.listener = None
        self.startTimer (1000)

    def timerEvent (self, event):
        self.ping ()
        
    def setListener (self, listener):
        """Set the listener, which will be called from ping in the
        main thread when a feed is loaded."""
        
        self.listener = listener

    def loadAll (self):
        for feed in feedDB ().getLiveFeeds ():
            self.load (feed)

       
    def load (self, feed):
        """Loads a feed asynchronously.  The loader's listener will be called
        from ping in the main thread when the feed is loaded. """
        
        #refuse to start a load on anything that's in the thread list or queue
        for q in self.queue:
            if q == feed:
                return
            
        for t in self.threads:
            if t.feed == feed:
                return
            
        if len (self.threads) > 3:
            self.queue.append (feed)
        else:
            thr = FeedLoaderThread (feed)
            self.threads.append (thr)
            thr.start ()

    def ping (self):
        
        """Should be called every N seconds *by the main thread*.  If
        any threads have finished, kill them and call the listener
        function.  If there's are fewer than three threads running,
        and there are any queued threads, start a queued thread.  If
        any feeds need to be updated, add them to the queue"""

        for feed in feedDB ().getLiveFeeds ():
            if feed.needsUpdate ():
                self.load (feed)

        threadList = self.threads [:]
        for thread in threadList:
            if not thread.isAlive ():
                #finished with this thread
                thread.join ()
                self.threads.remove (thread)
                if self.listener:
                    self.listener (thread.feed)

        while len (self.threads) <= 3 and len (self.queue):
            feed = self.queue.pop ()
            thr = FeedLoaderThread (feed)
            self.threads.append (thr)
            thr.start ()        

class FeedLoaderThread (Thread):
    def __init__ (self, feed):
        self.feed = feed
        Thread.__init__ (self)
        self.bogus = 0
        
    def run (self):
        try:
            self.feed.update ()
            feedDB ().store (self.feed)
        except (Exception), e:
            self.bogus = 1
            #FIXME: this needs real error handling
            #print e
