#!/usr/bin/python

# RSS Mix Tape
# Copyright 2003, David "Novalis" Turner <novalis@fsf.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

from xml.sax.saxutils import XMLGenerator

import time
import Config

from FeedDB2 import feedDB
from QuickTopic import createQuickTopic

class RSSGenerator:
    def generate (self):
        config = Config.config ()
        outputDir = config.get ("rss", "outputDir")
        files = {}
        outStreams = {}
        now = time.time ()
        
        for cat in config.getCategories ():
            files [cat] = open (outputDir + "/" + cat + ".rss", "w")
            out = outStreams [cat] = XMLGenerator (files [cat], 'utf-8')
            out.startDocument ()

            out.startElement ("rss", {'version' : '0.92'})
            out.startElement ("channel", {})

            for x in ("title", "link", "description", "managingEditor"):
                if config.get ("rss", x):
                    out.startElement (x, {})
                    out.characters (config.get ("rss", x))
                    out.endElement (x)

                out.startElement ("generator", {})
                out.characters ("RSS Mix Tape/" + config.get ("main", "version"))
                out.endElement ("generator")

                out.startElement ("ttl", {})
                out.characters (str (int (int (config.get ("rss", "minUpdate")) / 60)))
                out.endElement ("ttl")

        for post in feedDB ().posts ():
            if post.shouldGenerate ():
                out = outStreams [post.posted]
                out.startElement ("item", {})

                #does this need a quicktopic?

                if post.createQT and not post.comments:
                    post.comments = createQuickTopic (post.title)

                for attr in 'title', 'link', 'comments':
                    
                    out.startElement (attr, {})
                    out.characters (getattr (post, attr))
                    out.endElement (attr)

                out.startElement ("pubDate", {})
                strdate = time.asctime (time.localtime (post.date))
                out.characters (strdate)
                out.endElement ("pubDate")

                out.startElement ("category", {})
                out.characters (post.posted)
                out.endElement ("category")

                out.startElement ("description", {})
                out.characters (post.comment)
                out.characters ("<blockquote>")
                out.characters (post.text)
                out.characters ("</blockquote>")
                out.endElement ("description")

                out.endElement ("item")

                if not post.generated:
                    post.generated = int (now)

        for cat in config.getCategories ():
            out = outStreams [cat]
            out.endElement ("channel")
            out.endElement ("rss")

            out.endDocument ()
            files [cat].close ()
